"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const PakFormat_1 = require("./PakFormat");
const crypto_1 = __importDefault(require("crypto"));
class Mod {
    constructor(file) {
        this.file = file;
    }
}
exports.Mod = Mod;
class ModLoadOrder {
    constructor() {
        this.loadOrder = {};
    }
}
exports.ModLoadOrder = ModLoadOrder;
class ModManager {
    constructor() {
        this.mods = new Array();
        this.pakNames = [];
        this.pakFiles = [];
        this.folderNames = [];
        this.loPath = "./ModLoader/load_order.json";
        this.guiDataPath = "./user_mod_list.json";
        this.guiBaseDataPath = "./base_mod_list.json";
        this.order = new ModLoadOrder();
    }
    _recursive(parent, child, order) {
        let paks = new Array();
        let dir = path_1.default.join(parent, child);
        if (!fs_1.default.existsSync(dir)) {
            return;
        }
        fs_1.default.readdirSync(dir).forEach((file) => {
            if (fs_1.default.lstatSync(path_1.default.join(dir, file)).isDirectory()) {
                this.folderNames.push(file);
                console.log(file);
                this._recursive(dir, file, order);
            }
            let parse = path_1.default.parse(file);
            if (parse.ext === '.pak') {
                let modPak = new PakFormat_1.Pak(path_1.default.join(dir, parse.base));
                if (modPak.verify()) {
                    paks.push(modPak);
                }
                else {
                    console.log("THIS PAK (" + file + ") is corrupt!");
                }
            }
            else if (parse.ext === '.bps' ||
                parse.base.indexOf('.bps.disabled') > -1) {
                let patch = new Mod(path_1.default.join(dir, parse.base));
                patch.category = '_patches';
                let icon = '';
                if (fs_1.default.existsSync('./resources/flips.png')) {
                    icon = fs_1.default.readFileSync('./resources/flips.png').toString('base64');
                }
                else {
                    console.log(path_1.default.resolve('./resources/app/flips.png'));
                    icon = fs_1.default
                        .readFileSync('./resources/app/flips.png')
                        .toString('base64');
                }
                patch.meta = {
                    name: file.replace('.bps', '').replace('.disabled', ''),
                    version: '',
                };
                patch.icon = icon;
                patch.hash = crypto_1.default.createHash('md5').update(fs_1.default.readFileSync(patch.file)).digest('hex');
                patch.parentfolder = path_1.default.parse(parent).base;
                patch.subfolder = child;
                this.mods.push(patch);
            }
        });
        paks.forEach((modPak) => {
            if (!order.loadOrder.hasOwnProperty(modPak.fileName)) {
                order.loadOrder[path_1.default.parse(modPak.fileName).base] = false;
            }
            console.log(path_1.default.parse(modPak.fileName).base);
            this.pakNames.push(path_1.default.parse(modPak.fileName).base);
            let mod = new Mod(modPak.fileName);
            this.pakFiles.push(mod.file);
            for (let i = 0; i < modPak.pak.header.files.length; i++) {
                if (modPak.pak.header.files[i].filename.indexOf('package.json') > -1) {
                    let meta = JSON.parse(modPak.load(i).toString());
                    mod.meta = meta;
                    break;
                }
            }
            for (let i = 0; i < modPak.pak.header.files.length; i++) {
                if (modPak.pak.header.files[i].filename.indexOf('icon.png') > -1) {
                    let icon = modPak.load(i);
                    mod.icon = icon.toString('base64');
                    mod.type = 'png';
                    break;
                }
                else if (modPak.pak.header.files[i].filename.indexOf('icon.gif') > -1) {
                    let icon = modPak.load(i);
                    mod.icon = icon.toString('base64');
                    mod.type = 'gif';
                    break;
                }
            }
            if (mod.meta.hasOwnProperty("isBPS")) {
                if (mod.meta.isBPS) {
                    mod.category = "_patches";
                }
            }
            mod.hash = crypto_1.default.createHash('md5').update(fs_1.default.readFileSync(mod.file)).digest('hex');
            mod.parentfolder = path_1.default.parse(parent).base;
            mod.subfolder = child;
            this.mods.push(mod);
        });
    }
    reinitLoadOrder() {
        this.order = new ModLoadOrder();
        return this.order;
    }
    scanMods(parent, child) {
        if (fs_1.default.existsSync(this.loPath)) {
            console.log("Loading load order from file.");
            this.order = JSON.parse(fs_1.default.readFileSync(this.loPath).toString());
        }
        this._recursive(parent, child, this.order);
        if (fs_1.default.existsSync(this.guiDataPath)) {
            this.guiData = JSON.parse(fs_1.default.readFileSync(this.guiDataPath).toString());
        }
        if (fs_1.default.existsSync(this.guiBaseDataPath)) {
            this.baseguiData = JSON.parse(fs_1.default.readFileSync(this.guiBaseDataPath).toString());
        }
        Object.keys(this.order.loadOrder).forEach((key) => {
            if (this.pakNames.indexOf(key) === -1) {
                delete this.order.loadOrder[key];
            }
        });
        this.saveLoadOrder();
    }
    saveLoadOrder() {
        fs_1.default.writeFileSync(this.loPath, JSON.stringify(this.order, null, 2));
    }
}
exports.ModManager = ModManager;
//# sourceMappingURL=ModManager.js.map