'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const electron_1 = require("electron");
const electron_util_1 = require("electron-util");
const electron_unhandled_1 = __importDefault(require("electron-unhandled"));
const child_process_1 = require("child_process");
const MessageLayer_1 = require("./MessageLayer");
const GUITunnel_1 = require("./GUITunnel");
const ModManager_1 = require("./ModManager");
const fs_1 = __importDefault(require("fs"));
const RomManager_1 = require("./RomManager");
const discord_1 = require("./discord/discord");
const ModLoaderErrorCodes_1 = require("./ModLoaderErrorCodes");
const ModLoader64GUIConfig_1 = require("./ModLoader64GUIConfig");
require('source-map-support').install();
electron_unhandled_1.default();
//debug();
//contextMenu();
// Note: Must match `build.appId` in package.json
electron_1.app.setAppUserModelId('com.hylianmodding.modloader64-gui');
// Prevent window from being garbage collected
let status;
let loadingWindow;
let mainWindow;
let ModLoader64;
let updateProcess;
let transitionTimer;
let runningWindow;
let mods;
let roms;
let discord;
let rom = '';
let inputConfigChild;
const HIDE_BARS = true;
class NodeSideMessageHandlers {
    constructor(emitter, retriever) {
        this.layer = new MessageLayer_1.MessageLayer('internal_event_bus', emitter, retriever);
        this.layer.setupMessageProcessor(this);
    }
    onSetup(obj) { }
    async onStart(values) {
        console.log(values);
        let configPath = path_1.default.resolve(path_1.default.join('./ModLoader', 'ModLoader64-config.json'));
        let config = JSON.parse(fs_1.default.readFileSync(configPath).toString());
        config['NetworkEngine.Client'].nickname = values.nickname;
        config['NetworkEngine.Client'].lobby = values.lobby;
        config['NetworkEngine.Client'].password = values.password;
        config['ModLoader64'].rom = values.rom;
        console.log(values);
        if (values.isOffline || values.selfhost) {
            config['NetworkEngine.Client'].isSinglePlayer = true;
        }
        if (!values.isOffline && !values.selfhost) {
            config['NetworkEngine.Client'].isSinglePlayer = false;
        }
        config['NetworkEngine.Client'].ip = values.serverIP;
        config['NetworkEngine.Client'].port = values.serverPort;
        config['NetworkEngine.Client'].forceServerOverride = values.alternateConnection;
        rom = values.rom;
        let found = false;
        let patches = [];
        this.findBPS(values.data, patches);
        for (let i = 0; i < patches.length; i++) {
            let parse = path_1.default.parse(patches[i]);
            if (parse.ext === '.bps') {
                console.log(parse.base);
                config['ModLoader64'].patch = parse.base;
                found = true;
            }
        }
        if (!found) {
            config['ModLoader64'].patch = '';
        }
        fs_1.default.writeFileSync(configPath, JSON.stringify(config, null, 2));
        startModLoader();
    }
    onModStatusChanged(data) {
        fs_1.default.writeFileSync("./user_mod_list.json", JSON.stringify(data, null, 2));
        this.findEnabledMods(data, mods.reinitLoadOrder());
        mods.saveLoadOrder();
    }
    findEnabledMods(root, order) {
        if (root.hasOwnProperty("children")) {
            let children = root.children;
            for (let i = 0; i < children.length; i++) {
                this.findEnabledMods(children[i], order);
            }
        }
        if (root.hasOwnProperty("_checked")) {
            order.loadOrder[path_1.default.parse(root.attributes.file).base] = root._checked;
        }
    }
    findBPS(root, patches) {
        if (root.hasOwnProperty("children")) {
            let children = root.children;
            for (let i = 0; i < children.length; i++) {
                this.findBPS(children[i], patches);
            }
        }
        if (root.hasOwnProperty("_checked")) {
            if (root._checked) {
                patches.push(root.attributes.file);
            }
        }
    }
    onModListLoaded(data) {
        fs_1.default.writeFileSync("./base_mod_list.json", JSON.stringify(data, null, 2));
    }
    onInputConfigClick(evt) {
        let p = "\"" + path_1.default.resolve('./ModLoader/InputConfigTool.exe') + "\"";
        if (process.platform.trim() !== 'win32') {
            p = p.replace('.exe', '');
        }
        console.log(p);
        if (inputConfigChild !== null && inputConfigChild !== undefined) {
            return;
        }
        inputConfigChild = child_process_1.exec(p, {
            cwd: path_1.default.resolve('./ModLoader'),
        }, (error) => {
            if (error) {
                console.log(error);
            }
        });
        inputConfigChild.on('exit', (code) => {
            console.log(code);
            inputConfigChild = undefined;
        });
    }
    onFlips(evt) {
        let p = "\"" + path_1.default.resolve('./ModLoader/flips.exe') + "\"";
        if (process.platform.trim() !== 'win32') {
            p = p.replace('.exe', '');
        }
        let child = child_process_1.exec(p, {
            cwd: path_1.default.resolve('./ModLoader'),
        }, (error) => {
            if (error) {
                console.log(error);
            }
        });
        child.on('exit', (code) => {
            console.log("Trying to refresh mods tab...");
            mods = new ModManager_1.ModManager();
            mods.scanMods('./ModLoader', 'mods');
            handlers.layer.send('readMods', mods);
        });
    }
    onrefreshMods(evt) {
        console.log("Trying to refresh mods tab...");
        mods = new ModManager_1.ModManager();
        mods.scanMods('./ModLoader', 'mods');
        handlers.layer.send('readMods', mods);
    }
    onrefreshRoms(evt) {
        console.log("Trying to refresh games tab...");
        roms = new RomManager_1.RomManager();
        roms.getRoms();
        handlers.layer.send('readRoms', roms);
    }
    onForwardToML(evt) {
        ModLoader64.send(JSON.stringify(new GUITunnel_1.GUITunnelPacket('forwardToML', evt.id, evt)));
    }
    async onmodBrowser(evt) {
        const win = new electron_1.BrowserWindow({
            title: electron_1.app.name,
            show: false,
            width: 1024,
            height: 768,
            webPreferences: {
                nodeIntegration: true,
            },
            frame: true,
        });
        win.on('ready-to-show', () => {
            win.show();
            mod_browser_handlers = new ModBrowserHandlers(win.webContents, electron_1.ipcMain);
            mod_browser_handlers.layer.send("readMods", mods);
        });
        await win.loadFile(path_1.default.join(__dirname, 'mod_browser.html'));
    }
}
__decorate([
    GUITunnel_1.TunnelMessageHandler('electronSetup')
], NodeSideMessageHandlers.prototype, "onSetup", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('onStartButtonPressed')
], NodeSideMessageHandlers.prototype, "onStart", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('onModStatusChanged')
], NodeSideMessageHandlers.prototype, "onModStatusChanged", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('onModListLoaded')
], NodeSideMessageHandlers.prototype, "onModListLoaded", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('onInputConfig')
], NodeSideMessageHandlers.prototype, "onInputConfigClick", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('onFlips')
], NodeSideMessageHandlers.prototype, "onFlips", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('refreshMods')
], NodeSideMessageHandlers.prototype, "onrefreshMods", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('refreshRoms')
], NodeSideMessageHandlers.prototype, "onrefreshRoms", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler('forwardToML')
], NodeSideMessageHandlers.prototype, "onForwardToML", null);
__decorate([
    GUITunnel_1.TunnelMessageHandler("modBrowser")
], NodeSideMessageHandlers.prototype, "onmodBrowser", null);
class RunningWindowHandlers {
    constructor(emitter, retriever) {
        this.layer = new MessageLayer_1.MessageLayer('internal_event_bus', emitter, retriever);
        this.layer.setupMessageProcessor(this);
    }
}
class ModBrowserHandlers {
    constructor(emitter, retriever) {
        this.layer = new MessageLayer_1.MessageLayer('internal_event_bus', emitter, retriever);
        this.layer.setupMessageProcessor(this);
    }
}
let handlers;
let running_handlers;
let loading_handlers;
let mod_browser_handlers;
const createLoadingWindow = async () => {
    const win = new electron_1.BrowserWindow({
        title: electron_1.app.name,
        show: false,
        width: 400,
        height: 220,
        webPreferences: {
            nodeIntegration: true,
        },
        frame: false,
    });
    loading_handlers = new RunningWindowHandlers(win.webContents, electron_1.ipcMain);
    win.on('ready-to-show', () => {
        win.show();
        console.log('TRYING TO UPDATE MODLOADER');
        loading_handlers.layer.send("onLoadingStep", "Updating ModLoader...");
        updateProcess = child_process_1.fork(__dirname + '/updateModLoader.js');
        updateProcess.on('exit', (code, signal) => {
            console.log('TRYING TO UPDATE PLUGINS');
            loading_handlers.layer.send("onLoadingStep", "Updating cores...");
            updateProcess = child_process_1.fork(__dirname + '/updateCores.js');
            updateProcess.on('exit', (code, signal) => {
                loading_handlers.layer.send("onLoadingStep", "Updating mods...");
                updateProcess = child_process_1.fork(__dirname + '/updatePlugins.js');
                updateProcess.on('exit', (code, signal) => {
                    if (!fs_1.default.existsSync('./ModLoader/ModLoader64-config.json')) {
                        updateProcess = null;
                        return;
                    }
                    console.log('TRYING TO UPDATE GUI');
                    loading_handlers.layer.send("onLoadingStep", "Updating launcher...");
                    updateProcess = child_process_1.fork(__dirname + '/updateGUI.js');
                    updateProcess.on('exit', (code, signal) => {
                        console.log('GUI UPDATE CODE ' + code);
                        if (code == 1852400485) {
                            electron_1.app.relaunch();
                            electron_1.app.exit();
                        }
                        updateProcess = null;
                    });
                });
            });
        });
    });
    win.on('closed', () => {
        loadingWindow = undefined;
    });
    await win.loadFile(path_1.default.join(__dirname, 'loading.html'));
    return win;
};
const createRunningWindow = async () => {
    const win = new electron_1.BrowserWindow({
        title: electron_1.app.name,
        show: false,
        width: 600,
        height: 400,
        x: mainWindow.x,
        y: mainWindow.y,
        webPreferences: {
            nodeIntegration: true,
        },
        frame: false,
    });
    win.on('ready-to-show', () => {
        win.show();
    });
    await win.loadFile(path_1.default.join(__dirname, 'running.html'));
    win.setPosition(mainWindow.getPosition()[0], mainWindow.getPosition()[1]);
    running_handlers = new RunningWindowHandlers(win.webContents, electron_1.ipcMain);
    return win;
};
const createMainWindow = async () => {
    const win = new electron_1.BrowserWindow({
        title: "",
        show: false,
        width: 600,
        height: 400,
        webPreferences: {
            nodeIntegration: true,
        },
    });
    if (!fs_1.default.existsSync("./ModLoader64-GUI-config.json")) {
        fs_1.default.writeFileSync("./ModLoader64-GUI-config.json", JSON.stringify(new ModLoader64GUIConfig_1.ModLoader64GUIConfig(), null, 2));
    }
    let cfg = new ModLoader64GUIConfig_1.ModLoader64GUIConfig().fromFile(JSON.parse(fs_1.default.readFileSync("./ModLoader64-GUI-config.json").toString()));
    fs_1.default.writeFileSync("./ModLoader64-GUI-config.json", JSON.stringify(cfg, null, 2));
    electron_1.globalShortcut.register(cfg.keybindings.soft_reset.key, () => {
        let evt = { id: "SOFT_RESET_PRESSED", data: {} };
        if (ModLoader64 !== undefined || ModLoader64 !== null) {
            if (!ModLoader64.killed) {
                ModLoader64.send(JSON.stringify(new GUITunnel_1.GUITunnelPacket('forwardToML', evt.id, evt)));
            }
        }
    });
    win.on('ready-to-show', () => {
        transitionTimer = setInterval(() => {
            if (loadingWindow && updateProcess == null) {
                discord = new discord_1.DiscordIntegration();
                mods = new ModManager_1.ModManager();
                mods.scanMods('./ModLoader', 'mods');
                roms = new RomManager_1.RomManager();
                roms.getRoms();
                clearInterval(transitionTimer);
                handlers.layer.send('readMods', mods);
                handlers.layer.send('readRoms', roms);
                handlers.layer.send('readGUIConfig', cfg);
                if (!fs_1.default.existsSync(path_1.default.join('./ModLoader', 'ModLoader64-config.json'))) {
                    // Need to generate the config.
                    startModLoader();
                    setInterval(() => {
                        if (ModLoader64 !== null && status === 'onPostInitDone') {
                            ModLoader64.kill();
                        }
                        if (ModLoader64 === null) {
                            electron_1.app.relaunch();
                            electron_1.app.exit();
                        }
                    }, 1000);
                }
                else {
                    // We have a config.
                    console.log('Loading config.');
                    let config = JSON.parse(fs_1.default
                        .readFileSync(path_1.default.join('./ModLoader', 'ModLoader64-config.json'))
                        .toString());
                    console.log(config);
                    handlers.layer.send('onConfigLoaded', config);
                    console.log("Loading Mupen config...");
                    if (fs_1.default.existsSync(path_1.default.join(".", "ModLoader", "emulator", "mupen64plus.cfg"))) {
                        let mupen = fs_1.default.readFileSync(path_1.default.join(".", "ModLoader", "emulator", "mupen64plus.cfg")).toString();
                        let lines = mupen.split("\n");
                        let opts = {};
                        for (let i = 0; i < lines.length; i++) {
                            if (lines[i].indexOf("[") > -1) {
                                continue;
                            }
                            if (lines[i].indexOf("#") > -1) {
                                continue;
                            }
                            if (lines[i].trim() === "") {
                                continue;
                            }
                            let s = lines[i].split("=");
                            opts[s[0].trim()] = s[1].trim().replace(/['"]+/g, "");
                        }
                        console.log(JSON.stringify(opts, null, 2));
                    }
                }
                loadingWindow.close();
                if (!fs_1.default.existsSync('./ModLoader/src/version.js')) {
                    electron_1.dialog.showErrorBox("ModLoader64 has crashed!", "Failed to install ModLoader core files. Restart this program to enter repair mode.");
                    if (fs_1.default.existsSync("./update.json")) {
                        fs_1.default.unlinkSync("./update.json");
                    }
                    electron_1.app.exit();
                }
                win.setTitle("ModLoader64-GUI" +
                    ' ' +
                    electron_1.app.getVersion() +
                    ' | ' +
                    'ModLoader64 ' +
                    require(path_1.default.resolve('./ModLoader/src/version')));
                if (HIDE_BARS) {
                    win.removeMenu();
                }
                win.show();
            }
        }, 1000);
    });
    win.on('closed', () => {
        mainWindow = undefined;
    });
    handlers = new NodeSideMessageHandlers(win.webContents, electron_1.ipcMain);
    await win.loadFile(path_1.default.join(__dirname, 'index.html'));
    return win;
};
electron_1.app.on('window-all-closed', () => {
    if (!electron_util_1.is.macos) {
        electron_1.app.quit();
    }
});
electron_1.app.on('activate', () => {
    if (!mainWindow) {
        mainWindow = createMainWindow();
    }
});
const API_WINDOWS = new Array();
(async () => {
    await electron_1.app.whenReady();
    if (HIDE_BARS) {
        electron_1.Menu.setApplicationMenu(null);
    }
    loadingWindow = await createLoadingWindow();
    mainWindow = await createMainWindow();
})();
function apiHandler(evt) {
    switch (evt.event) {
        case 'openWindow':
            let win = new electron_1.BrowserWindow({
                title: electron_1.app.name,
                show: false,
                width: evt.data[0].width,
                height: evt.data[0].height,
                webPreferences: {
                    nodeIntegration: true,
                },
            });
            win.on('ready-to-show', () => {
                win.removeMenu();
                win.show();
                win.setParentWindow(runningWindow);
            });
            win.loadFile(evt.data[0].file);
            API_WINDOWS.push(new MessageLayer_1.MessageLayer('modloader64_api_window', win.webContents, electron_1.ipcMain));
            break;
        case 'setDiscordStatus':
            discord.setActivity(evt.data[0]);
            break;
    }
}
async function startModLoader() {
    discord.setActivity({ details: 'Loading a game', state: rom });
    mainWindow.hide();
    API_WINDOWS.length = 0;
    if (runningWindow === null || runningWindow === undefined) {
        runningWindow = await createRunningWindow();
    }
    const options = {
        stdio: ['pipe', 'pipe', 'pipe', 'ipc'],
    };
    console.log(path_1.default.resolve('./ModLoader/src/index.js'));
    ModLoader64 = child_process_1.fork('./ModLoader/src/index.js', ['--dir=./ModLoader'], options);
    ModLoader64.on('message', (message) => {
        let evt = JSON.parse(message);
        if (evt.id === 'internal_event_bus') {
            handlers.layer.send('onStatus', evt.event);
            status = evt.event;
        }
        else if (evt.id === 'modloader64_api') {
            apiHandler(evt);
        }
        else {
            handlers.layer.send(evt.event, evt.data[0]);
            for (let i = 0; i < API_WINDOWS.length; i++) {
                try {
                    API_WINDOWS[i].send(evt.event, evt.data[0]);
                }
                catch (err) { }
            }
        }
    });
    ModLoader64.on('exit', (code) => {
        console.log(code);
        if (code !== 0 && code < 100 && code !== null) {
            electron_1.dialog.showErrorBox("ModLoader64 has crashed!", ModLoaderErrorCodes_1.ModLoaderErrorCodes[code]);
        }
        try {
            mainWindow.setPosition(runningWindow.getPosition()[0], runningWindow.getPosition()[1]);
            runningWindow.close();
            runningWindow = null;
            mainWindow.show();
            ModLoader64 = null;
            discord.reset();
        }
        catch (err) { }
    });
    ModLoader64.stdout.on('data', (buf) => {
        let msg = buf.toString();
        if (msg === '' || msg === null || msg === undefined) {
            return;
        }
        running_handlers.layer.send('onLog', msg);
        console.log(msg);
    });
}
//# sourceMappingURL=index.js.map