"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const PakFormat_1 = require("./PakFormat");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const request_1 = __importDefault(require("request"));
const download = require('download-file');
let url = require('url');
let isDev = fs_1.default.existsSync("./DEV_FLAG.json");
function getAllFiles(dirPath, arrayOfFiles) {
    let files = fs_1.default.readdirSync(dirPath);
    arrayOfFiles = arrayOfFiles || [];
    files.forEach((file) => {
        if (fs_1.default.statSync(dirPath + "/" + file).isDirectory()) {
            arrayOfFiles = getAllFiles(dirPath + "/" + file, arrayOfFiles);
        }
        else {
            arrayOfFiles.push(path_1.default.join(dirPath, "/", file));
        }
    });
    return arrayOfFiles;
}
// Check plugins.
let updatedSomething = false;
if (fs_1.default.existsSync('./ModLoader/mods')) {
    let download_dir = "./downloads";
    let cores_dir = "./ModLoader/cores";
    let mods_dir = "./ModLoader/mods";
    if (!fs_1.default.existsSync(download_dir)) {
        fs_1.default.mkdirSync(download_dir);
    }
    getAllFiles(mods_dir, []).forEach((file) => {
        let parse = path_1.default.parse(file);
        if (parse.ext === ".disabled") {
            fs_1.default.renameSync(file, path_1.default.join(parse.dir, parse.name));
            file = path_1.default.join(parse.dir, parse.name);
            parse = path_1.default.parse(parse.name);
        }
        if (parse.ext === '.pak') {
            console.log('Found .pak file: ' + parse.base + '.');
            let modPak = new PakFormat_1.Pak(file);
            if (modPak.verify()) {
                for (let i = 0; i < modPak.pak.header.files.length; i++) {
                    if (modPak.pak.header.files[i].filename.indexOf('package.json') > -1) {
                        let meta = JSON.parse(modPak.load(i).toString());
                        if (meta.hasOwnProperty('modloader64_deps')) {
                            Object.keys(meta['modloader64_deps']).forEach((key) => {
                                console.log(meta.name + ' requires ' + key + '.');
                                if (!fs_1.default.existsSync(path_1.default.join('./ModLoader/cores', key + '.pak'))) {
                                    let repo = meta['modloader64_deps'][key];
                                    repo = repo.replace(".git", "");
                                    repo = repo.replace('https://github.com', 'https://raw.githubusercontent.com') + '/master/update.json';
                                    console.log(repo);
                                    request_1.default(repo, (error, response, body) => {
                                        if (!error && response.statusCode === 200) {
                                            let resp = JSON.parse(body);
                                            let options = {
                                                directory: download_dir,
                                                filename: path_1.default.basename(url.parse(resp.url).pathname),
                                            };
                                            console.log(resp);
                                            download(resp.url, options, function (err) {
                                                if (err)
                                                    throw err;
                                                let pak = new PakFormat_1.Pak(path_1.default.join(download_dir, path_1.default.basename(url.parse(resp.url).pathname)));
                                                if (pak.verify()) {
                                                    console.log("Pak file verified.");
                                                    fs_1.default.copyFileSync(path_1.default.join(download_dir, path_1.default.basename(url.parse(resp.url).pathname)), path_1.default.join(cores_dir, path_1.default.basename(url.parse(resp.url).pathname)));
                                                    fs_1.default.unlinkSync(path_1.default.join(download_dir, path_1.default.basename(url.parse(resp.url).pathname)));
                                                }
                                                console.log('Finished updating ' + meta.name + '.');
                                                updatedSomething = true;
                                            });
                                        }
                                    });
                                }
                            });
                        }
                        if (meta.hasOwnProperty('updateUrl')) {
                            let updateurl = meta.updateUrl;
                            if (isDev && meta.hasOwnProperty("devUrl")) {
                                updateurl = meta.devUrl;
                            }
                            console.log(updateurl);
                            console.log(meta);
                            request_1.default(updateurl, (error, response, body) => {
                                if (!error && response.statusCode === 200) {
                                    const resp = JSON.parse(body);
                                    let pversion = meta.version;
                                    if (pversion !== resp.version) {
                                        console.log('Updating ' + meta.name + '...');
                                        console.log(resp);
                                        let options = {
                                            directory: download_dir,
                                            filename: parse.base,
                                        };
                                        let download_url = resp.url;
                                        if (resp.hasOwnProperty("devurl")) {
                                            download_url = resp.devurl;
                                        }
                                        download(download_url, options, function (err) {
                                            if (err)
                                                throw err;
                                            let pak = new PakFormat_1.Pak(path_1.default.join(download_dir, path_1.default.basename(url.parse(resp.url).pathname)));
                                            if (pak.verify()) {
                                                console.log("Pak file verified.");
                                                fs_1.default.copyFileSync(path_1.default.join(download_dir, path_1.default.basename(url.parse(resp.url).pathname)), path_1.default.join(mods_dir, path_1.default.basename(url.parse(resp.url).pathname)));
                                                fs_1.default.unlinkSync(path_1.default.join(download_dir, path_1.default.basename(url.parse(resp.url).pathname)));
                                            }
                                            console.log('Finished updating ' + meta.name + '.');
                                            updatedSomething = true;
                                        });
                                    }
                                    else {
                                        console.log('No update needed: ' + meta.name + '.');
                                    }
                                }
                            });
                        }
                        else {
                            console.log('No update entry found.');
                        }
                        break;
                    }
                }
            }
            else {
                fs_1.default.unlinkSync(path_1.default.join('./ModLoader/mods', parse.base));
            }
        }
    });
}
//# sourceMappingURL=updatePlugins.js.map